unit _Fractions;

interface
Uses _LongIntegers;
Type
TFrac = record
    sign	:Integer; // -1 for negitive 1 for positive
    whole	:TInt;
    numerator 	:TInt;
    denominator	:TInt;
  end;

TFracL = record
    whole	:TInt;
    numerator 	:TInt;
    denominator	:TInt;
  end;


function AddFracF(val1,val2:TFrac):TFrac;
function AddFracL(val1,val2:TFracL):TFracL;
Function SubFracF(frac1, frac2:TFrac):TFrac;
Function MulFracF(f1, f2:TFrac):TFrac;
Function DivFracF(f1, f2:TFrac):TFrac;

function MakeImproper(f:TFracL):TfracL; overload;
Function MakeImproper(f:TFrac):TFrac ; overload;
function MakeProper(f:TFrac):Tfrac; overload;
function MakeProper(f:TFracL):TfracL; overload;

function Reduce(f:TFrac):Tfrac;         overload;
Function Reduce(f:TFracL): TFracL;	overload;
Function GreatestCommonDivisor(n1, n2:TInt):TInt;
Function FindLowestCommonMultiple(n1, n2 :TInt): TInt;

Function DivF(Dividend, divisor:TLongInt): TFrac;

Function FracLToFrac(f:TFracL; sign :Tsign = 1):TFrac;
Function FracToFracL(f:TFrac):TFracL;


implementation
const
  properFractions :Boolean = True;
  improperFractions :Boolean = False;

Function DivF(Dividend, divisor:TLongInt): TFrac;
var
  q,r:TLongInt;
begin
  q := DivI(Dividend, divisor, r);
  result.sign := q.sign;
  result.whole := q.int;
  result.numerator := r.int;
  result.denominator := divisor.int;
  result := Reduce(result);
end;



Function Reduce(f:TFrac): TFrac;	overload;
begin
  result := FracLToFrac(Reduce(FracToFracL(f)),f.sign)
end;

Function Reduce(f:TFracL): TFracL;	overload;
var
  GCD: TInt;
begin
  GCD := GreatestCommonDivisor(f.numerator, f.denominator);
  result.whole := f.whole;
  result.numerator := DivL(f.numerator,GCD);
  result.denominator := DivL(f.denominator, GCD);
end;

Function GreatestCommonDivisor(n1, n2:TInt):TInt;
var
  t:Tint;
begin
  t:=nil;
  if CompL(n1, n2) < 0 then
  begin
    t := n1;
    n1 := n2;
    n2 := t;
  end;
  if CompLZ(n2) then
  begin
    SetLength(result,1);
    result[0] := 1;
  end
  else begin
    while not CompLZ(n2) do
    begin
      t := n2;
      divL(n1,n2,n2);
      n1 := t;
  //    gcd(N,M) = gcd(M, N mod M)
    end;
    result := n1;
  end;
end;

Function FindLowestCommonMultiple(n1, n2 :TInt): TInt;
var
  gcd:TInt;
begin
  gcd := GreatestCommonDivisor(n1,n2);
  Result := MulL(n1,n2);
  Result := DivL(result, gcd);
end;

Function MakeImproper(f:TFrac):TFrac ; overload;
begin
  result := FracLToFrac(MakeImproper(FracToFracL(f)),f.sign);
end;

Function MakeImproper(f:TFracL):TFracL ; overload;
begin
  result.whole := nil;
  result.numerator := MulL(f.denominator,f.whole);
  result.numerator := AddL(result.numerator,f.numerator);
  result.denominator := f.denominator;
end;

Function MakeProper(f:TFrac):TFrac ;
begin
  result := FracLToFrac(MakeProper(FracToFracL(f)),f.sign);
end;

Function MakeProper(f:TFracL):TFracL ;
begin
  result.whole := divL(f.numerator, f.denominator, result.numerator);
  result.whole := AddL(result.whole, f.whole);
  result.denominator := f.denominator;
end;

Function FracToFracL(f:TFrac):TFracL;
//The sign is lost
begin
  result.whole := f.whole;
  result.numerator := f.numerator;
  result.denominator := f.denominator;
end;

Function FracLToFrac(f:TFracL; sign :Tsign = 1):TFrac;
begin
  result.sign := sign;
  result.whole := f.whole;
  result.numerator := f.numerator;
  result.denominator := f.denominator;
end;


Function SubFracF(frac1, frac2:TFrac):TFrac;
var
  LCM :TInt;
begin
  if (frac1.sign = -1) and (frac2.sign = -1) then
  begin
    frac1.sign := 1;
    frac2.sign := 1;
    result := SubFracF(frac2,frac1);
  end
  else if (frac1.sign = -1) and (frac2.sign <> -1) then
  begin
    result := FracLToFrac(AddFracL(FracToFracL(frac1), FracToFracL(frac2)), -1);
  end
  else if (frac1.sign <> -1) and (frac2.sign = -1) then
  begin
    result := FracLToFrac(AddFracL(FracToFracL(frac1), FracToFracL(frac2)), 1);
  end;
  LCM := FindLowestCommonMultiple(frac1.denominator, frac2.denominator);
  frac1.numerator := MulL(frac1.numerator, DivL(LCM, frac1.denominator));
  frac2.numerator := MulL(frac2.numerator, DivL(LCM, frac2.denominator));
  frac1.denominator := LCM;
  frac2.denominator := LCM;
  if CompL(frac1.numerator, frac2.numerator) >= 0 then
  begin
      result.whole := SubI(frac1.whole, frac2.whole, result.sign);
      frac1.numerator := SubL(frac1.numerator, frac2.numerator);
      result.numerator := frac1.numerator;
      result.denominator := frac1.denominator;
  end
  else begin
     frac1 := MakeImproper(frac1);
     frac2 := MakeImproper(frac2);
     if CompL(frac1.numerator, frac2.numerator) < 0 then
     begin
       frac2.numerator := SubL(frac2.numerator, frac1.numerator);
       frac1.numerator := frac2.numerator;
       frac1.sign := -1;
     end
     else frac1.numerator := SubL(frac1.numerator, frac2.numerator);
     result := frac1;
  end;
  result := Reduce(result);
  if properFractions then result := MakeProper(result)
  else if improperFractions then result := MakeImproper(result);
end;

function AddFracF(val1,val2:TFrac):TFrac;
//var
//  n1, n2, m1, m2 :TInt;
begin
  if (val1.sign = -1) and (val2.sign = -1) then result.sign := -1
  else if (val1.sign = -1) and (val2.sign <> -1) then
  begin
    val1.sign := 1;
    result := SubFracF(val2,val1);
    exit;
  end
  else if (val1.sign <> -1) and (val2.sign = -1) then
  begin
    val2.sign := 1;
    result := SubFracF(val1,val2);
    exit;
  end;
  result := FracLToFrac(AddFracL(FracToFracL(val1), FracToFracL(val2)), 1);
end;

function AddFracL(val1,val2:TFracL):TFracL;
var
  commonMult,m1,m2, n1, n2:TInt;
begin
  result.whole := AddL(val1.whole, val2.whole );
  commonMult := FindLowestCommonMultiple(val1.denominator, val2.denominator);
  m1 := divL(commonMult,val1.denominator);
  m2 := divL(commonMult,val2.denominator);
  n1 := MulL(val1.numerator, m1);
  n2 := MulL(val2.numerator, m2);
  n1 := AddL(n1,n2);
  result.numerator := n1;
  result.denominator := commonMult;
  result := Reduce(result);
  if properFractions then result := MakeProper(result)
  else if improperFractions then result := MakeImproper(result);
end;

Function MulFracF(f1, f2:TFrac):TFrac;
begin
  f1 := Reduce(f1);
  f2 := Reduce(f2);
  f1 := MakeImproper(f1);
  f2 := MakeImproper(f2);
  result.whole := nil;
  result.whole := mulL(f1.whole,f2.whole);
  result.numerator := MulL(f1.numerator,f2.numerator);
  result.denominator := MulL(f1.denominator,f2.denominator);
  result.sign := f1.sign;
  if f2.sign = -1 then result.sign := result.sign * -1;
  if properFractions then result := MakeProper(result)
  else if improperFractions then result := MakeImproper(result);
end;

Function DivFracF(f1, f2:TFrac):TFrac;
begin
  f1 := Reduce(f1);
  f2 := Reduce(f2);
  f1 := MakeImproper(f1);
  f2 := MakeImproper(f2);
  result.whole := nil;
  result.numerator := MulL(f1.numerator,f2.denominator);
  result.denominator := MulL(f1.denominator,f2.numerator);
  result.sign := f1.sign;
  if f2.sign = -1 then result.sign := result.sign * -1;
  if properFractions then result := MakeProper(result)
  else if improperFractions then result := MakeImproper(result);
end;

end.
