unit _LongFloat;
interface
Uses _LongIntegers;
type
  TLongFloat = record
	Mantissa:		TLongInt;
	exponential:		LongInt;
	SignificantDigits:	LongWord;	//The maximum number of longwords to use for the Mantissa
  end;

Function AddFloat(n1,n2: TlongFloat): TlongFloat;
Function SubFloat(n1,n2: TLongFloat): TLongFloat;
Function MultFloat(n1,n2:TlongFloat): TlongFloat;
Function DivFloat(n1,n2:TlongFloat): TlongFloat;
Function CompLF(n1,n2:TlongFloat): Integer;
Function CompLFAbs(const n1,n2:TlongFloat): Integer;
Function Absfloat(n1:TlongFloat): TlongFloat;

Function _AddFloat(n1,n2:TlongFloat): TlongFloat;
Function _SubFloat(n1,n2:TlongFloat): TlongFloat;

implementation
uses _Mem;

Function Absfloat(n1:TlongFloat): TlongFloat;
begin
  result := n1;
  result.Mantissa.sign := 1;
end;

Procedure ZeroLF(var n1: TlongFloat);
begin
  setLength(n1.Mantissa.int,0);
  n1.Mantissa.sign := 0;
  n1.exponential := 0;
end;

Function CompLFAbs(const n1,n2:TlongFloat): Integer;
var
  x,l1,l2:LongWord;
begin
  l1:= lengthUsed(n1.Mantissa.int);
  l2:= lengthUsed(n2.Mantissa.int);
  if l1 = 0 then
  begin
    if l2 = 0 then result := 0
    else result := -1
  end
  else if l2 = 0 then result := 1
  else if n1.exponential > n2.exponential then result := 1
  else if n1.exponential < n2.exponential then result := -1
  else begin
    if l1 = l2 then result := CompL(n1.Mantissa.int,n2.Mantissa.int)
    else begin
      x:=1;
      while (x <= l1) and (x <= l2) do
      begin
	if n1.Mantissa.int[l1-x] = n2.Mantissa.int[l2-x] then inc(x)
	else begin
	  if n1.Mantissa.int[l1-x] > n2.Mantissa.int[l2-x] then result := 1
	  else result := -1;
	  exit;
	end;
      end;
      if l1 > l2 then result := 1
      else result := -1
    end;
  end;
end;

Function CompLF(n1,n2:TlongFloat): Integer;
begin
  if (LengthUsed(n1.Mantissa.int) = 0) and (n1.Mantissa.sign < 0) then n1.Mantissa.sign := 1;
  if (LengthUsed(n2.Mantissa.int) = 0) and (n2.Mantissa.sign < 0) then n2.Mantissa.sign := 1;
  if n1.Mantissa.sign < 0 then
  begin
    if n2.Mantissa.sign < 0 then result := CompLFAbs(n1,n2)
    else result := - 1;
  end
  else begin
    if n2.Mantissa.sign >= 0 then result := CompLFAbs(n1,n2)
    else result := 1;
  end;
end;

Function DivFloat(n1,n2:TlongFloat): TlongFloat;
var
  t: TLongInt;
begin
  result.SignificantDigits := n1.SignificantDigits;
  result.Mantissa := DivI(n1.Mantissa,n2.Mantissa,t);
  while (not CompLZ(t.int))
  and (lengthUsed(result.Mantissa.int) < result.SignificantDigits) do
  begin
    while (compL(t.int,n2.Mantissa.int) = -1)
    and (lengthUsed(result.Mantissa.int) < result.SignificantDigits) do
    begin
      Shunt32(result.Mantissa.int);
      Shunt32(t.int);
    end;
    result.Mantissa.int := AddL(result.Mantissa.int,DivI(t,n2.Mantissa,t).int);
  end;
  result.exponential :=
  (n1.exponential - (LongInt(LengthUsed(n1.Mantissa.int))-1))
  - (n2.exponential - (LongInt(LengthUsed(n2.Mantissa.int))-1))
  + (LongInt(LengthUsed(result.Mantissa.int))-1);
end;

Function MultFloat(n1,n2:TlongFloat): TlongFloat;
begin
  result.SignificantDigits := n1.SignificantDigits;
  result.Mantissa := MulI(n1.Mantissa,n2.Mantissa);
  result.exponential :=
  (n1.exponential - (LongInt(LengthUsed(n1.Mantissa.int))-1))
  + (n2.exponential - (LongInt(LengthUsed(n2.Mantissa.int))-1))
  + (LongInt(LengthUsed(result.Mantissa.int)-1));
  if Length(result.Mantissa.int) > LongInt(result.SignificantDigits)
  then TruncI(result.Mantissa, result.SignificantDigits);
end;

Function LeastSignificant(n1:TLongFloat): LongInt;
//Least Significant Digits pos on numberline
var
  lu: LongInt;
begin
  lu := LengthUsed(n1.Mantissa.int);
  if lu > 0 then result := n1.exponential - (lu) + 1
  else result := 0;
end;

Function MostSignificant(n1:TLongFloat): LongInt;
begin
  result := n1.exponential
end;

Function AddShift(n1,n2:TLongFloat): LongInt;
// The amount that n2 will nead to be shifted for the least snifficant
// digits to line up.
// negitive value inicates n1 will nead to be added starting at
// -(result) in n2
begin
  result := LeastSignificant(n2)-LeastSignificant(n1)
end;

Function FloatOverlap(n1,n2:TLongFloat): Boolean;
begin
  result :=
    (
      (MostSignificant(n1) <= MostSignificant(n2))
      and
      (MostSignificant(n2) <= LeastSignificant(n1))
    )
    or
    (
      (MostSignificant(n1) <= LeastSignificant(n2))
      and
      (LeastSignificant(n2)<= LeastSignificant(n1))
    )
end;

Function Gap(n1,n2:TLongFloat): LongInt;
// negitive values indicate overlap.
begin
  if n1.exponential >= n2.exponential then
    result := LeastSignificant(n1) - MostSignificant(n2) -1
  else
    result := LeastSignificant(n2) - MostSignificant(n1) -1;
end;

Function AddFloat(n1,n2:TLongFloat): TLongFloat;
begin
  result.SignificantDigits := n1.SignificantDigits;
  if LengthUsed(n1.Mantissa.int) = 0 then result := n2
  else if LengthUsed(n2.Mantissa.int) < 1 then result := n1
  else if n1.Mantissa.sign >= 0 then
  begin
    if n2.Mantissa.sign >= 0 then
    begin
      result := _AddFloat(n1,n2);
      result.Mantissa.sign := 1;
    end
    else begin
      n2.Mantissa.sign := 1;
      result := SubFloat(n1,n2);
    end;
  end
  else begin
    if n2.Mantissa.sign >= 0 then
    begin  // n1 < 0 and n2 >= 0
      n1.Mantissa.sign := 1;
      result := SubFloat(n2,n1);
    end
    else begin
      result := _AddFloat(n1,n2);
      result.Mantissa.sign := -1;
    end;
  end;
end;

Function SubFloat(n1,n2: TLongFloat): TLongFloat;
var
  x:Integer;
begin
  result.SignificantDigits := n1.SignificantDigits;
  if LengthUsed(n1.Mantissa.int) = 0 then
  begin
    result := n2;
    if result.Mantissa.sign >= 0 then result.Mantissa.sign := -1
    else result.Mantissa.sign := 1;
  end
  else if LengthUsed(n2.Mantissa.int) < 1 then result := n1
  else if n1.Mantissa.sign >= 0 then
  begin
    if n2.Mantissa.sign >= 0 then
    begin
      x := CompLFAbs(n1,n2);
      if x > 0 then
      begin
	result := _SubFloat(n1,n2);
	result.Mantissa.sign := 1;
      end
      else if x < 0 then
      begin
	result := _SubFloat(n2,n1);
	result.Mantissa.sign := -1;
      end
      else begin
	ZeroLF(result);
	result.Mantissa.sign := 0;
      end;
    end
    else begin //n1 > 0, n2 < 0
      result := _AddFloat(n1,n2);
      result.Mantissa.sign := 1;
    end;
  end
  else begin
    if n2.Mantissa.sign >= 0 then
    begin  // n1 < 0 and n2 >= 0
      result := _AddFloat(n1,n2);
      result.Mantissa.sign := -1;
    end
    else begin // n1 < 0, n2 < 0
      n1.Mantissa.sign := 1;
      n2.Mantissa.sign := 1;
      result := SubFloat(n1,n2);
      if result.Mantissa.sign < 0 then result.Mantissa.sign := 1
      else result.Mantissa.sign := -1;
    end;
  end;
end;

Function _AddFloat(n1,n2:TLongFloat): TLongFloat;
// no Mantissa.int with a length of 0; (nil not checked for)
// all numbers will be treated as positive
var
  x, l1, l2, g:	LongInt;
  t:		TLongFloat;
begin
  result.SignificantDigits := n1.SignificantDigits;
  l1 := LengthUsed(n1.Mantissa.int);
  l2 := LengthUsed(n2.Mantissa.int);
  if FloatOverlap(n1,n2) then
  begin
    x := AddShift(n1,n2);
    if x > 0 then
    begin
      result.Mantissa.int := copy(n1.Mantissa.int);
      AddLAt(result.Mantissa.int,n2.Mantissa.int,x);
      if (n1.Mantissa.int[l1-1] = bit32)
      and (result.Mantissa.int[lengthUsed(result.Mantissa.int)-1] <> Bits32)
      then result.exponential := n1.exponential + 1
      else result.exponential := n1.exponential;
    end
    else if x < 0 then
    begin
      result.Mantissa := n2.Mantissa;
      SetLength(result.Mantissa.int,length(result.Mantissa.Int));
      AddLAt(result.Mantissa.int,n1.Mantissa.int,-x);
      if (n2.Mantissa.int[l1-1] = bit32)
      and (result.Mantissa.int[lengthUsed(result.Mantissa.int)-1] <> Bits32)
      then result.exponential := n2.exponential + 1
      else result.exponential := n2.exponential;
    end
    else begin
      result.Mantissa.int := AddL(n1.Mantissa.int,n2.Mantissa.int);
      x := LengthUsed(result.Mantissa.int);
      if l1 > l2 then
      begin
	if n1.Mantissa.int[l1-1] = result.Mantissa.int[x-1]
	then result.exponential := n1.exponential
	else result.exponential := n1.exponential + 1;
      end
      else if  l1 < l2 then
      begin
	if n2.Mantissa.int[l1-1] = result.Mantissa.int[x-1]
	then result.exponential := n2.exponential
	else result.exponential := n2.exponential + 1;
      end
      else if x > l1 then result.exponential := n1.exponential + 1
      else result.exponential := n1.exponential;
    end;
  end
  else begin
    if n2.exponential > n1.exponential then
    begin
      t := n1;
      n1 := n2;
      n2 := t;
    end;
    result.exponential := n1.exponential;
    g := Gap(n1,n2);
    if l1+g+l2 > LongInt(result.SignificantDigits) then SetLength(result.Mantissa.int,result.SignificantDigits)
    else SetLength(result.Mantissa.int, l1+g+l2);
    x:= length(result.Mantissa.int);{$WARN UNSAFE_CODE Off}
    if l1 > x then CopyMem32(addr(n1.Mantissa.int[0]), addr(result.Mantissa.int[0]), x)
    else begin
      CopyMem32(addr(n1.Mantissa.int[0]), addr(result.Mantissa.int[0]), l1);
      if l1+g > x then SetMem32(addr(result.Mantissa.int[l1]),x-l1)
      else begin
	SetMem32(addr(result.Mantissa.int[l1]),g);
	if l1+g+l2 > x then CopyMem32(addr(n2.Mantissa.int[0]), addr(result.Mantissa.int[l1+g]), x-(l1+g))
	else CopyMem32(addr(n2.Mantissa.int[0]), addr(result.Mantissa.int[l1+g]), l2);
      end;
    end;
  end;
end;{$WARN UNSAFE_CODE ON}

Function _SubFloat(n1,n2:TlongFloat): TlongFloat;
// n1 must be >= n2
// no Mantissa.int with a length of 0;
// all numbers will be treated as positive
var
  x,  g,discard, l1, l2 :LongInt;
//	:LongWord;
begin
  result.SignificantDigits := n1.SignificantDigits;
  l1 := LengthUsed(n1.Mantissa.int);
  l2 := LengthUsed(n2.Mantissa.int);
  if FloatOverlap(n1,n2) then
  begin
    x := AddShift(n1,n2);
    if x > 0 then
    begin
      g := gap(n1,n2);//g will be negitive giving the overlap;
      if LongInt(l1+l2)+g > LongInt(result.SignificantDigits) then discard := (LongInt(l1+l2)+g) - LongInt(result.SignificantDigits)
      else discard := 0;
      SetLength(result.Mantissa.int,LongInt(l1)+(LongInt(l2)-discard)+g); {$WARN UNSAFE_CODE Off} //g=-overlap
      CopyMem32(n1.Mantissa.int
	,addr(result.Mantissa.int[length(result.Mantissa.int)-(l2+g)]),l1);
      SetMem32(result.Mantissa.int, length(result.Mantissa.int)-(l2+g-discard));
      _SubL(result.Mantissa.int, addr(n2.Mantissa.int[discard]),l2-discard);{$WARN UNSAFE_CODE ON}
      if result.Mantissa.int[length(result.Mantissa.int)] = 0 then
	result.exponential := n1.exponential - 1
      else result.exponential := n1.exponential;
    end
    else begin //x = 0 here
      result.Mantissa.int := Copy(n1.Mantissa.int);
      _subL(result.Mantissa.int, n2.Mantissa.int, l2);
      if lengthUsed(result.Mantissa.int) < LongWord(l1) then result.exponential := n1.exponential
      else result.exponential := n1.exponential -1;
    end;
  end
  else begin //if not FloatOverlap(n1,n2)
    g := Gap(n1,n2);
    if l1+g >= longInt(result.SignificantDigits) then result.Mantissa := n1.Mantissa
    else if l1+g+l2 > longInt(result.SignificantDigits) then
    begin
      SetLength(result.Mantissa.int,result.SignificantDigits);
      x := l1+g+l2 - longInt(result.SignificantDigits);   {$WARN UNSAFE_CODE Off}
      SetMem32(addr(result.Mantissa.int[0]),g+(l2-x));
      CopyMem32(addr(n1.Mantissa.int[0]),addr(result.Mantissa.int[(l2-x)+g]),l1);
      _SubL(result.Mantissa.int,addr(n2.Mantissa.int[x]),l2-x);
    end
    else begin
      SetLength(result.Mantissa.int,l1+g+l2);
      SetMem32(addr(result.Mantissa.int[0]),g+l2);
      CopyMem32(addr(n1.Mantissa.int[0]),addr(result.Mantissa.int[l2+g]),l1);
      _SubL(result.Mantissa.int,n2.Mantissa.int,l2);
    end;{$WARN UNSAFE_CODE ON}
  end
end;

end.
