unit soundDB;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, Grids, DBGrids, DBTables, Constdef, MPlayer, IBDatabase,
  IBCustomDataSet, IBTable, Menus, _ScoreUtils2;

type
  TSoundsForm = class(TForm)
    DBGrid1: TDBGrid;
    MainMenu1: TMainMenu;
    Sounddb1: TMenuItem;
    Addsound1: TMenuItem;
    Delete1: TMenuItem;
    Findsound1: TMenuItem;
    GetFile: TOpenDialog;
    IBTable1: TIBTable;
    MenuExit: TMenuItem;
    FindDialog1: TFindDialog;

    procedure WriteWord(f:string);
    procedure PlayWord(s:string);
    procedure Addsound1Click(Sender: TObject);
    procedure DBGrid1ColEnter(Sender: TObject);
    procedure DBGrid1CellClick(Column: TColumn);
    Function WavForWord(word:String):Boolean;
    procedure Delete1Click(Sender: TObject);
    function ShowModal: Integer; reintroduce;
    procedure DBGrid1KeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure MenuExitClick(Sender: TObject);
    procedure Findsound1Click(Sender: TObject);
    procedure FindDialog1Find(Sender: TObject);

  private
    { Private declarations }
    find: String;
    Procedure AddWordToDB;
    procedure DeleteRow;
  public
    { Public declarations }
  end;

var
  SoundsForm: TSoundsForm;
  tableChanged:	Boolean = false;

implementation
uses MMSystem,_Dos, _SqlUtills;

{$R *.DFM}

function TSoundsForm.ShowModal: Integer;
begin
  if pas.PasLevel >= listSoundsPasLev then
    begin
      SU.ConectDB;
      su.IBDataSet1.close;
      su.IBDataSet1.SelectSQL.Text := 'select * from sound';
      su.IBDataSet1.open;
      while not su.IBDataSet1.Eof do su.IBDataSet1.Next;
      su.IBDataSet1.First;
      tableChanged := False;
      su.IBTransaction1.CommitRetaining;
      result := inherited ShowModal;
      if tableChanged then
	begin
	  if mrYes = MessageDlg('Data has been deleted commit changes'
	    , mtConfirmation, [mbYes, mbNo], 0)
	  then su.IBTransaction1.CommitRetaining
	  else
	    su.IBTransaction1.Rollback;
	    SU.ConectDB;
	end
    end
    else begin
      result := 0;
      MessageDlg('Access denied',  mtWarning, [mbOK],0);
    end;
end;

Procedure TSoundsForm.AddWordToDB;
var
  x :Word;
begin
  if GetFile.Execute then          { Display Open dialog box }
  begin
     for x := 0 to getFile.Files.Count - 1 do
      WriteWord(getFile.Files[x]);   { File selected in dialog box }
  end;
end;

{$WARN UNSAFE_CODE OFF} {$WARN UNSAFE_TYPE OFF}
procedure TSoundsForm.PlayWord(s:String);
var
  buf		:pchar;
  size	:integer;
  bs		:TStream;
begin
  SU.ConectDB;
  IBTable1.Active := true;
   if not IBTable1.Locate('name', s, [loCaseInsensitive]) then exit;
  bs := SoundsForm.IBTable1.CreateBlobStream(IBTable1.FieldByName('data'), bmRead);
  size := bs.Size;
  getmem(buf, Size);
  bs.Read(buf^,size);
  PlaySound(buf, 0,  SND_MEMORY+ SND_ASYNC);
  bs.Free;
  FreeMem(Buf);
end;{$WARN UNSAFE_CODE ON} {$WARN UNSAFE_TYPE ON}

function TSoundsForm.WavForWord(word:String):Boolean;
begin
  try
    SU.ConectDB;
    IBTable1.Active := true;
     if IBTable1.Locate('name', word, [loCaseInsensitive]) then
      begin
	Result := True;
      end
      else begin
	Result := False;
      end;
  except
    Result := False;
  end;
end;

{$WARN UNSAFE_TYPE OFF} {$WARN UNSAFE_CODE Off}
procedure TSoundsForm.WriteWord(f:string);
var
  file1:file;
  buf:			pchar;
  size, NumRead:	integer;
  bs:  			TStream;
begin
  SU.ConectDB;
  IBTable1.Active := true;
  try
     if IBTable1.Locate('name', fileName(f), [loCaseInsensitive]) then
      begin
	 if mrYes <> MessageDlg('"'+fileName(f)+'" Word allready in database overwrite! ',
		    mtConfirmation, [mbYes, mbNo], 0) then exit;
	IBTable1.Edit;
      end
      else begin
	IBTable1.Last;
	IBTable1.Insert;
      end;
  except
    exit;
  end;
  try
    AssignFile(file1, f);
    Reset(File1, 1);	{ Record size = 1 }
    size := FileSize(file1);
    getmem(buf, Size);
  except
    exit;
  end;
  try
    BlockRead(file1, Buf^, size, NumRead);
    CloseFile(file1);
    bs := SoundsForm.IBTable1.CreateBlobStream(IBTable1.FieldByName('data'),bmWrite);
  except
    FreeMem(Buf);
    exit;
  end;
  try
    bs.Write(buf^,size);
    IBTable1.FieldByName('name').asString := fileName(f);
  finally
    FreeMem(Buf);
    bs.Free;
  end;
  IBTable1.Post;
  su.IBTransaction1.CommitRetaining;
end;{$WARN UNSAFE_TYPE ON} {$WARN UNSAFE_CODE ON}

procedure TSoundsForm.Addsound1Click(Sender: TObject);
begin
  AddWordToDB;
end;

procedure TSoundsForm.DBGrid1ColEnter(Sender: TObject);
begin
  PlayWord(DBGrid1.DataSource.DataSet.FieldByName('name').asString);
end;

procedure TSoundsForm.DBGrid1CellClick(Column: TColumn);
begin
  PlayWord(DBGrid1.DataSource.DataSet.FieldByName('name').asString);
end;

procedure TSoundsForm.Delete1Click(Sender: TObject);
begin
  DeleteRow;
end;

procedure TSoundsForm.DeleteRow;
var
  name, sql:	 String;
begin
  name := DBGrid1.DataSource.DataSet.FieldByName('name').asString;
  sql := 'Delete from sound where name = ';
  AddSqlQuote(sql, name);
  if pas.PasLevel < soundDelPasLev
    then MessageDlg('Access denied',  mtWarning, [mbOK],0)
  else if mrYes = MessageDlg('Delete word '+ name + ' from sound database ? '
      , mtConfirmation, [mbYes, mbNo], 0)
    then begin
      su.IBDataSet1.close;
      su.IBDataSet1.DeleteSQL.Text :=  sql;
      su.IBDataSet1.open;
      DBGrid1.DataSource.DataSet.Delete;
      tableChanged := true;
    end;
end;

procedure TSoundsForm.DBGrid1KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_DELETE then DeleteRow;
  if Key = VK_INSERT then AddWordToDB;
end;

procedure TSoundsForm.MenuExitClick(Sender: TObject);
begin
   SoundsForm.ModalResult := mrOk;
end;

procedure TSoundsForm.Findsound1Click(Sender: TObject);
begin
  find := '';
  DBGrid1.DataSource.DataSet.Filtered := False;
  FindDialog1.Execute;
end;

procedure TSoundsForm.FindDialog1Find(Sender: TObject);
var
  findS: String;
  ops:	TLocateOptions;
begin
with Sender as TFindDialog do
  begin
  ops := [];
  if not (frMatchCase in Options) then ops := ops + [loCaseInsensitive];
  if not (frWholeWord in Options) then ops := ops + [loPartialKey];
  if FindText = '' then exit;
  if find = FindText then
    begin
      su.IBDataSet1.Next;
      if not su.IBDataSet1.LocateNext('name', find, ops) then
	begin
	  su.IBDataSet1.Prior;
	  ShowMessage('Search string <' + find + '> not found');
	end;
    end
    else begin
      find := FindText;
      findS := 'name = '#39 + find + #39;
      if not su.IBDataSet1.Locate('name', find, ops) then
	ShowMessage('Search string <' + find + '> not found');
    end;
  if not FindDialog1.Execute then Exit;
  end
end;

end.
